#include "config.h"
#include "echo.h"

void
str_cli(int fd, int sockfd)
{
   size_t n;
   char sendline[MAXLINE], recvline[MAXLINE];

   while ( (n = Readline(fd, sendline, MAXLINE))) {
      writen(sockfd, sendline, strlen(sendline));
      if (Readline(sockfd, recvline, MAXLINE) == 0)
         ERR_QUIT ("str_cli: server terminated prematurely");
      Writen (1, "echo: ", 6);
      Writen (1, recvline, n);
   }
}

void
str_echo(int sockfd)
{
   ssize_t  n;
   char     line[MAXLINE];
   pid_t    self;

   self = getpid();

   for ( ; ; ) {
      if ( (n = Readline(sockfd, line, MAXLINE)) == 0) {
         return;      /* connection closed by other end */
      }
      fprintf (stderr, "(server pid = %d)\n", (int) self);
      Writen (sockfd, line, n);
   }
}

static ssize_t
my_read(int fd, char *ptr)
{
   static int   read_cnt = 0;
   static char *read_ptr;
   static char  read_buf[MAXLINE];

   if (read_cnt <= 0) {
again:
      if ( (read_cnt = read(fd, read_buf, sizeof(read_buf))) < 0) {
         if (errno == EINTR) goto again;
         return(-1);
      } else if (read_cnt == 0) return(0);
      read_ptr = read_buf;
   }
   read_cnt--;
   *ptr = *read_ptr++;
   return (1);
}

ssize_t
readline(int fd, void *vptr, size_t maxlen)
{
   int      n, rc;
   char   c, *ptr;

   ptr = vptr;
   for (n = 1; n < maxlen; n++) {
      if ( (rc = my_read(fd, &c)) == 1) {
         *ptr++ = c;
         if (c == '\0') break;
      } else if (rc == 0) {
         if (n == 1) return(0); /* EOF, no data read */
         else break;            /* EOF, some data was read */
      } else return(0);        /* error, errno set by read() */
      /* what happens if we return -1 ? */
   }

   *ptr = 0;   /* null terminate like fgets() */
   return(n);
}

ssize_t
Readline(int fd, void *ptr, size_t maxlen)
{
   ssize_t      n;
   if ( (n = readline(fd, ptr, maxlen)) < 0)
      ERR_QUIT ("readline error");
   return(n);
}

ssize_t 
writen(int fd, const void *vptr, size_t n)
/* Write "n" bytes to a descriptor. */
{
   size_t      nleft;
   ssize_t     nwritten;
   const char *ptr;

   ptr = vptr;
   nleft = n;
   while (nleft > 0) {
      if ( (nwritten = write(fd, ptr, nleft)) <= 0) {
         if (errno == EINTR) nwritten = 0;  /* and call write() again */
         else return(-1);        /* error */
      }
      nleft -= nwritten;
      ptr   += nwritten;
   }
   return(n);
}

void
Writen (int fd, void *ptr, size_t nbytes)
{
   if (writen(fd, ptr, nbytes) != nbytes)
      ERR_QUIT ("writen error");
}








